<?php if ( ! defined('BASEPATH')) exit('No direct script access allowed');

require_once PATH_THIRD.'mountee_3/config.php';
require_once PATH_THIRD.'mountee_3/lib/EE_templates_lib.php';
require_once PATH_THIRD.'mountee_3/lib/Mountee_lib.php';
require_once PATH_THIRD.'mountee_3/lib/Mountee_settings_lib.php';
require_once PATH_THIRD.'mountee_3/lib/MLC_Api.php';

class Helper
{
	var $ee_template_lib = null;
	var $mountee_lib = null;

	public function __construct()
	{
		$this->ee_template_lib = new EE_templates_lib(ee()->config->item('site_id'));
		$this->mountee_lib = new Mountee_lib();
	}

	public function get_status()
	{
		$message = null;
		if (ee()->config->item('save_tmpl_files') == 'y')
		{
			$status = 'disabled';
			$message = 'You can\'t use Mountee when save template as files is enabled';
		}
		elseif (ee()->config->item('save_tmpl_files') == 'n')
		{
			$status = 'ok';
		}
		else
		{
			$status = 'disabled';
			$message = 'Error checking templates saved as files';
		}

		// Check licensing
		if ($this->mountee_lib->check_license())
		{
			$license_ok = TRUE;
		}
		else
		{
			$license_ok = FALSE;
		}

		$response_content = array(
			'cms'			=> 'ExpressionEngine',
			'cms_version'	=> 3,
			'addon_version'	=> MOUNTEE_3_VERSION,
			'status'		=> $status,
			'license'		=> $license_ok
		);

		if ($message != null)
		{
			$response_content['message'] = $message;
		}
		ee()->output->send_ajax_response($response_content);
	}

	/**
	 *
	 */
	public function login()
	{
		// If they are already logged in then send them away.
		if (ee()->session->userdata('member_id') !== 0)
		{
			$response = array(
				'logged_in' => true,
				'permissions' => (ee()->session->userdata('can_access_design') == 'y'),
			);

			if (ee()->config->item('website_session_type') == 's' || ee()->config->item('website_session_type') == 'cs')
			{
				// Add session id in response as it's needed for auth
				$response['session_id'] = ee()->session->userdata('session_id');
			}
			ee()->output->send_ajax_response($response);
		}

		ee()->load->library('auth');

		/* -------------------------------------------
		/* 'member_member_login_start' hook.
		/*  - Take control of member login routine
		/*  - Added EE 1.4.2
		*/
			//ee()->extensions->call('member_member_login_start');
			//if (ee()->extensions->end_script === TRUE) return;
		/*
		/* -------------------------------------------*/

		// Figure out how many sites we're dealing with here
		$sites = ee()->config->item('multi_login_sites');
		$sites_array = explode('|', $sites);

		// No username/password?  Bounce them...
		$multi = (ee()->input->get('multi') && count($sites_array) > 0)
			? ee()->input->get('multi')
			: FALSE;
		$username = ee()->input->post('username');
		$password = ee()->input->post('password');

		if ( ! $multi && ! ($username && $password))
		{
			return ee()->output->send_ajax_response(array(
				'logged_in' => FALSE,
				'message' => 'missing auth info',
			));
		}

		if (strlen($password) > PASSWORD_MAX_LENGTH)
		{
			return ee()->output->send_ajax_response(array(
				'logged_in' => FALSE,
				'message' => 'password too short',
			));
		}

		// This should go in the auth lib.
		if ( ! ee()->auth->check_require_ip())
		{
			// Not sure what this is
			return ee()->output->send_ajax_response(array(
				'logged_in' => FALSE,
				'message' => 'require ip failed',
			));
		}

		// Check password lockout status
		if (TRUE === ee()->session->check_password_lockout($username))
		{
			return ee()->output->send_ajax_response(array(
				'logged_in' => FALSE,
				'message' => 'password lockout in effet',
			));
		}

		$success = '';

		// Log me in.
		if ($multi)
		{
			// Multiple Site Login
			$incoming = $this->_do_multi_auth($sites, $multi);
			$success = '_build_multi_success_message';

			$current_url = ee()->functions->fetch_site_index();
			$current_search_url = preg_replace('/\/S=.*$/', '', $current_url);
			$current_idx = array_search($current_search_url, $sites_array);
		}
		else
		{
			// Regular Login
			$incoming = $this->_do_auth($username, $password);
			$success = '_build_success_message';

			$current_url = ee()->functions->fetch_site_index();
			$current_search_url = preg_replace('/\/S=.*$/', '', $current_url);
			$current_idx = array_search($current_search_url, $sites_array);
		}

		// Set login state
		if ($multi)
		{
			$login_state = $multi;
		}
		else
		{
			$login_state = random_string('md5');
			ee()->db->update(
				'sessions',
				array('login_state' => $login_state),
				array('session_id' => ee()->session->userdata('session_id'))
			);
		}

		// More sites?
		if ($sites && ee()->config->item('allow_multi_logins') == 'y')
		{
			$this->_redirect_next_site($sites, $current_idx, $current_url, $login_state);
		}

		// Clear out login_state
		ee()->db->set('login_state', NULL)
			->where('session_id', ee()->session->userdata('session_id'))
			->update('sessions');

		// $this->$success($sites_array);

		// Load member data in session (needed otherwise permissions are not set !)
		ee()->session->fetch_member_data();

		// The permissions only checks for access to the Template Manager, 
		//  other checks will occur later because of the granularity
		$response = array(
			'logged_in' => true,
			'permissions' => (ee()->session->userdata('can_access_design') == 'y'),
		);

		if (ee()->config->item('website_session_type') == 's' || ee()->config->item('website_session_type') == 'cs')
		{
			// Add session id in response as it's needed for auth
			$response['session_id'] = ee()->session->userdata('session_id');
		}
		return ee()->output->send_ajax_response($response);
	}

	public function logout()
	{
		// Kill the session and cookies
		ee()->db->where('site_id', ee()->config->item('site_id'))
			->where('ip_address', ee()->input->ip_address())
			->where('member_id', ee()->session->userdata('member_id'))
			->delete('online_users');

		ee()->session->destroy();

		ee()->input->delete_cookie('read_topics');

		return ee()->output->send_ajax_response(array(
			'logged_in' => false,
		));
	}

	/**
	 * Get all templates/template groups/snippets/global vars 
	 */
	public function list_all()
	{
		$this->_init_check();

		// Get all template groups/templates
		$t_groups = $this->ee_template_lib->get_template_groups_with_templates();

		$data = array();
		foreach($t_groups as $template_group)
		{
			// foreach group, convert it and its templates to an array
			//  only if the user has access to it
			$template_group_arr = $this->mountee_lib->convert_to_mountee_array($template_group);
			$data['folders'][] = $template_group_arr;
			
			// if($template_group->is_site_default=="y")
			// {
			// 	$data["index"] = (int)$template_group->group_id;
			// }
		}
		
		// Get all Snippets
		$snippets = $this->ee_template_lib->get_snippets();
		$global_snippets = $this->ee_template_lib->get_global_snippets();

		$data['folders'][] = $this->mountee_lib->create_snippets_object(array_merge($snippets->asArray(), $global_snippets->asArray()));

		// Get all Variables
		$variables = $this->ee_template_lib->get_global_variables();
		$data['folders'][] = $this->mountee_lib->create_variables_object($variables);

		// License check
		if (!$this->mountee_lib->check_license())
		{
			$data['files'][] = $this->mountee_lib->get_license_dummy_file();
		}


		$data['success'] = TRUE;

		return ee()->output->send_ajax_response(
			$data
		);
	}

	public function create_template_group()
	{
		$this->_init_check();

		$template_group_name = ee()->input->post('template_group_name', TRUE);

		if ($template_group_name == "")
		{
			// Send Bad request http code
			set_status_header(400);
			return ee()->output->send_ajax_response(array(
				'success' => FALSE,
				'message' => 'The template group name cannot be empty.',
			));
		}

		if ( ! preg_match("#^[a-zA-Z0-9_\-/]+$#i", $template_group_name))
		{
			return $this->_send_name_invalid('template group');
		}

		$template_groups = $this->ee_template_lib->get_template_groups();

		$max_group_order = 0;
		foreach ($template_groups as $template_group)
		{
			if ($template_group_name == $template_group->group_name)
			{
				// Send Conflict http code
				set_status_header(409);
				return ee()->output->send_ajax_response(array(
					'success' => FALSE,
					'message' => 'The template group name will conflict with an existing template group.',
				));
			}
			if ($template_group->group_order > $max_group_order)
			{
				$max_group_order = $template_group->group_order;
			}
		}

		$template_group = ee('Model')->make('TemplateGroup', array(
			'group_name'		=> $template_group_name,
			'group_order'		=> $max_group_order+1,
			'is_site_default'	=> 'n',
			'site_id'			=> ee()->config->item('site_id')
		));
		// This fails in EE 3.2.1 -- EE bug, not the add-on :D
		//$result = $template_group->validate();

		$template_group->save();
			return ee()->output->send_ajax_response(array(
				'success' => TRUE,
				'message' => 'The template group has been created.',
				'template_group' => $this->mountee_lib->convert_to_mountee_array($template_group)
			));

		if ($result->isValid())
		{
			$template_group->save();
			return ee()->output->send_ajax_response(array(
				'success' => TRUE,
				'message' => 'The template group has been created.',
				'template_group' => $this->mountee_lib->convert_to_mountee_array($template_group)
			));
		}
		else
		{
			return $this->_send_object_invalid($result, 'template group');
		}
	}

	/**
	 * Create a new template in a given template group (or snippet or golbal variable)
	 */
	public function create_template()
	{
		$this->_init_check();

		$template_group_id = ee()->input->post('template_group_id');
		$template_name = ee()->input->post('template_name', TRUE);
		$template_data = ee()->input->post('template_data', TRUE);

		$extension = pathinfo($template_name, PATHINFO_EXTENSION);
		$template_type = Mountee_lib::get_type_from_extension($extension);

		// Remove extension from template name
		$template_name = str_replace('.'.$extension, '', $template_name);

		// Added space ' ' in it because it's too restrictive for a file system
		if ( ! preg_match("#^[a-zA-Z0-9_\ \-/]+$#i", $template_name))
		{
			return $this->_send_name_invalid('template');
		}

		if ($template_group_id == 'snippets')
		{
			// Make sure we don't go further
			return $this->_create_snippet();
		}

		if ($template_group_id == 'variables')
		{
			return $this->_create_global_variable();
		}

		// Verify the template type
		if ($template_type == '')
		{
			// Send Bad request http code
			set_status_header(400);
			return ee()->output->send_ajax_response(array(
				'success' => FALSE,
				'message' => 'The template type cannot be empty.',
			));
		}
		if (!in_array($template_type, $this->ee_template_lib->get_template_types()))
		{
			// Send Bad request http code
			set_status_header(400);
			return ee()->output->send_ajax_response(array(
				'success' => FALSE,
				'message' => 'The template type \''.$template_type.'\' isn\'t valid.',
			));
		}

		// Get the template group and verify that the new template won't override an existing one
		$template_group = $this->ee_template_lib->get_template_group($template_group_id);
		
		if ($template_group == NULL)
		{
			return $this->_send_object_not_found('template group', $template_group_id);
		}

		foreach ($template_group->Templates as $template)
		{
			if ($template->template_name == $template_name)
			{
				// Send Conflict http code
				set_status_header(409);
				return ee()->output->send_ajax_response(array(
					'success' => FALSE,
					'message' => 'The template name will conflict with an existing template.',
				));
			}
		}

		$template_name_temp = $template_name;
		if (strpos($template_name, ' ') !== FALSE)
		{
			// We accept spaces for convenience, but validation test would fail
			// So we replace spaces with tabs and add the timestamp to be sure it's unique
			$template_name_temp = str_replace(' ', '_', $template_name) . time();
		}
		$template = ee('Model')->make('Template', array(
			'template_name' => $template_name_temp,
			'template_type' => $template_type,
			'group_id'		=> $template_group_id,
			'site_id'		=> ee()->config->item('site_id')
		));

		if ($template_data != "")
		{
			$template->template_data = base64_decode($template_data);
		}

		$result = $template->validate();
		if ($result->isValid())
		{
			// Change back to the real template name
			$template->template_name = $template_name;
			$template->save();
			return ee()->output->send_ajax_response(array(
				'success' => TRUE,
				'message' => 'The template has been created.',
				'template' => $this->mountee_lib->convert_to_mountee_array($template)
			));
		}
		else
		{
			return $this->_send_object_invalid($result, 'template');
		}
	}

	public function get_template()
	{
		$this->_init_check();

		$template_group_id = ee()->input->post('template_group_id');
		$template_id = intval(ee()->input->post('template_id'));

		if ($template_group_id == 'snippets')
		{
			$snippet = $this->ee_template_lib->get_snippet($template_id);
			if ($snippet)
			{
				return ee()->output->send_ajax_response(array(
					'template' => $this->mountee_lib->convert_to_mountee_array($snippet)
				));
			}
			else
			{
				return $this->_send_object_not_found('snippet', $template_id);
			}
		}
		else if ($template_group_id == 'variables')
		{
			$global_var = $this->ee_template_lib->get_global_variable($template_id);
			if ($global_var)
			{
				return ee()->output->send_ajax_response(array(
					'template' => $this->mountee_lib->convert_to_mountee_array($global_var)
				));
			}
			else
			{
				return $this->_send_object_not_found('global variable', $template_id);
			}
		}
		else
		{
			// Get the template, check if it's parent is the correct one
			$template = $this->ee_template_lib->get_template($template_id);

			if ($template)
			{
				if ($template->group_id == $template_group_id)
				{
					return ee()->output->send_ajax_response(array(
						'template' => $this->mountee_lib->convert_to_mountee_array($template)
					));
				}
				else
				{
					// Send Bad request http code
					set_status_header(400);
					return ee()->output->send_ajax_response(array(
						'success' => FALSE,
						'message' => 'That template doesn\'t belong to that template group',
					));
				}
			}
			else
			{
				return $this->_send_object_not_found('template', $template_id);
			}
		}
	}

	public function move_template()
	{
		$this->_init_check();

		$template_group_id_from = ee()->input->post('from_group_id');
		$template_group_id_to = ee()->input->post('to_group_id');
		$template_id = intval(ee()->input->post('template_id'));

		if ($template_group_id_from == $template_group_id_to)
		{
			// Given the way the app works, this means it's a file rename
			return $this->rename_template();
		}

		$template_name = NULL;
		// Check that the template exists in the from template group
		if ($template_group_id_from == 'snippets')
		{
			$template_snippet = $this->ee_template_lib->get_snippet($template_id);
			if ($template_snippet)
			{
				$template_name = $template_snippet->snippet_name;
			}
			else
			{
				return $this->_send_object_not_found('snippet', $template_id);
			}
		}
		else if ($template_group_id_from == 'variables')
		{
			$template_global_var = $this->ee_template_lib->get_global_variable($template_id);
			if ($template_global_var)
			{
				$template_name = $template_global_var->variable_name;
			}
			else
			{
				return $this->_send_object_not_found('global variable', $template_id);
			}
		}
		else
		{
			// Get the template, check if it's parent is the correct one
			$template_template = $this->ee_template_lib->get_template($template_id);
			if ($template_template)
			{
				if ($template_template->group_id == $template_group_id_from)
				{
					$template_name = $template_template->template_name;
				}
				else
				{
					// Send Bad request http code
					set_status_header(400);
					return ee()->output->send_ajax_response(array(
						'success' => FALSE,
						'message' => 'That template doesn\'t belong to that template group',
					));
				}
			}
			else
			{
				return $this->_send_object_not_found('template', $template_id);
			}
		}

		// Check that a template with the same name doesn't exist in the to template group
		if ($template_group_id_to == 'snippets')
		{
			$snippet = $this->ee_template_lib->get_snippet_from_name($template_name);
			if ($snippet)
			{
				set_status_header(400);
				return ee()->output->send_ajax_response(array(
					'success' => FALSE,
					'message' => 'A template with that name already exists in that group.',
				));
			}
		}
		else if ($template_group_id_to == 'variables')
		{
			$global_var = $this->ee_template_lib->get_global_variable_from_name($template_name);
			if ($global_var)
			{
				set_status_header(400);
				return ee()->output->send_ajax_response(array(
					'success' => FALSE,
					'message' => 'A template with that name already exists in that group.',
				));
			}
		}
		else
		{
			$template = $this->ee_template_lib->get_template_from_name($template_group_id_to, $template_name);
			if ($template)
			{
				set_status_header(400);
				return ee()->output->send_ajax_response(array(
					'success' => FALSE,
					'message' => 'A template with that name already exists in that group.',
				));
			}
		}

		// Make the change
		if ($template_group_id_to == 'snippets')
		{
			if (isset($template_template))
			{
				// Convert the inital template to a snippet
				$new_snippet = ee('Model')->make('Snippet', array(
					'snippet_name' => $template_template->template_name,
					'snippet_contents' => $template_template->template_data
				));
				// Delete the initial template
				$template_template->delete();
			}
			else if (isset($template_global_var))
			{
				// Convert the Global Var to a Snippet
				$new_snippet = ee('Model')->make('Snippet', array(
					'snippet_name' => $template_global_var->variable_name,
					'snippet_contents' => $template_global_var->variable_data
				));
				// Delete the initial global var
				$template_global_var->delete();
			}
			$new_snippet->save();

			return ee()->output->send_ajax_response(array(
				'success' => TRUE,
				'template'=> $this->mountee_lib->convert_to_mountee_array($new_snippet),
				'message' => 'The template has been moved'
			));
		}
		else if ($template_group_id_to == 'variables')
		{
			if (isset($template_template))
			{
				// Convert the inital template to a Global Var
				$new_global_var = ee('Model')->make('GlobalVariable', array(
					'variable_name' => $template_template->template_name,
					'variable_data' => $template_template->template_data
				));
				// Delete the initial template
				$template_template->delete();
			}
			else if (isset($template_snippet))
			{
				// Convert the inital template to a Global Var
				$new_global_var = ee('Model')->make('GlobalVariable', array(
					'variable_name' => $template_snippet->snippet_name,
					'variable_data' => $template_snippet->snippet_contents
				));
				// Delete the initial template
				$template_snippet->delete();
			}
			$new_global_var->save();
			
			return ee()->output->send_ajax_response(array(
				'success' => TRUE,
				'template'=> $this->mountee_lib->convert_to_mountee_array($new_global_var),
				'message' => 'The template has been moved'
			));
		}
		else
		{
			if (isset($template_template))
			{
				$template_template->group_id = $template_group_id_to;
				$template_template->save();

				return ee()->output->send_ajax_response(array(
					'success' => TRUE,
					'template'=> $this->mountee_lib->convert_to_mountee_array($template_template),
					'message' => 'The template has been moved'
				));
			}
			else
			{
				if (isset($template_global_var))
				{
					$new_template = ee('Model')->make('Template', array(
						'template_name' => $template_global_var->variable_name,
						'template_type' => 'webpage',
						'group_id'		=> $template_group_id_to,
						'site_id'		=> ee()->config->item('site_id'),
						'template_data' => $template_global_var->variable_data
					));
					$template_global_var->delete();
				}
				else if (isset($template_snippet))
				{
					$new_template = ee('Model')->make('Template', array(
						'template_name' => $template_snippet->snippet_name,
						'template_type' => 'webpage',
						'group_id'		=> $template_group_id_to,
						'site_id'		=> ee()->config->item('site_id'),
						'template_data' => $template_snippet->snippet_contents
					));
					$template_snippet->delete();
				}
				$new_template->save();

				return ee()->output->send_ajax_response(array(
					'success' => TRUE,
					'template'=> $this->mountee_lib->convert_to_mountee_array($new_template),
					'message' => 'The template has been moved'
				));
			}
		}

		// If we arrive here, there's been an issue somewhere, send back an error
		set_status_header(500);
		return ee()->output->send_ajax_response(array(
			'success' => FALSE,
			'message' => 'We couldn\'t move the template, an error occured.',
		));
	}

	/**
	 * Update an existing template
	 */
	public function update_template_data()
	{
		$this->_init_check();

		$template_group_id = ee()->input->post('template_group_id');
		$template_id = intval(ee()->input->post('template_id'));
		$template_data = ee()->input->post('template_data', TRUE);

		if ($template_group_id == 'snippets')
		{
			$snippet = $this->ee_template_lib->get_snippet($template_id);
			if ($snippet)
			{
				$snippet->snippet_contents = base64_decode($template_data);
				$snippet->save();
				return ee()->output->send_ajax_response(array(
					'success' => TRUE,
					'message' => 'The snippet has been saved',
					'template' => $this->mountee_lib->convert_to_mountee_array($snippet)
				));
			}
			else
			{
				return $this->_send_object_not_found('snippet', $template_id);
			}
		}
		else if ($template_group_id == 'variables')
		{
			$global_var = $this->ee_template_lib->get_global_variable($template_id);
			if ($global_var)
			{
				$global_var->variable_data = base64_decode($template_data);
				$global_var->save();
				return ee()->output->send_ajax_response(array(
					'success' => TRUE,
					'message' => 'The Global Variable has been saved',
					'template' => $this->mountee_lib->convert_to_mountee_array($global_var)
				));
			}
			else
			{
				return $this->_send_object_not_found('global variable', $template_id);
			}
		}
		else
		{
			$template_group_id = intval($template_group_id);
			$template_id = intval($template_id);

			// Get the template, check if it's parent is the correct one
			$template = $this->ee_template_lib->get_template($template_id);
			if ($template)
			{
				if ($template->group_id == $template_group_id)
				{
					$template->template_data = base64_decode($template_data);
					$template->save();

					if (bool_config_item('save_tmpl_revisions'))
					{
						// Create a template revision
						$version = ee('Model')->make('RevisionTracker');
						$version->Template = $template;
						$version->item_table = 'exp_templates';
						$version->item_field = 'template_data';
						$version->item_data = $template->template_data;
						$version->item_date = ee()->localize->now;
						$version->Author = $template->LastAuthor;
						$version->save();

						// Now, rotate template revisions based on 'max_tmpl_revisions' config item
						$versions = ee('Model')->get('RevisionTracker')
							->filter('item_id', $template->getId())
							->filter('item_field', 'template_data')
							->order('item_date', 'desc')
							->limit(ee()->config->item('max_tmpl_revisions'))
							->all();

						// Reassign versions and delete the leftovers
						$template->Versions = $versions;
						$template->save();
					}

					return ee()->output->send_ajax_response(array(
						'success' => TRUE,
						'message' => 'The template has been saved',
						'template' => $this->mountee_lib->convert_to_mountee_array($template)
					));
				}
				else
				{
					// Send Bad request http code
					set_status_header(400);
					return ee()->output->send_ajax_response(array(
						'success' => FALSE,
						'message' => 'That template doesn\'t belong to that template group',
					));
				}
			}
			else
			{
				return $this->_send_object_not_found('template', $template_id);
			}
		}
	}

	public function delete_template_group()
	{
		$this->_init_check();

		$template_group_id = ee()->input->post('template_group_id');

		$template_group = $this->ee_template_lib->get_template_group($template_group_id);

		if ($template_group)
		{
			// Delete each template
			foreach ($template_group->Templates as $template)
			{
				$template->delete();
			}
			$template_group->delete();

			return ee()->output->send_ajax_response(array(
				'success' => TRUE,
				'message' => 'The Template Group has been deleted',
			));
		}
		else
		{
			return $this->_send_object_not_found('template group', $template_group_id);
		}
	}

	public function delete_template()
	{
		$this->_init_check();

		$template_group_id = ee()->input->post('template_group_id');
		$template_id = intval(ee()->input->post('template_id'));

		if ($template_group_id == 'snippets')
		{
			$snippet = $this->ee_template_lib->get_snippet($template_id);
			if ($snippet)
			{
				$snippet->delete();
				return ee()->output->send_ajax_response(array(
					'success' => TRUE,
					'message' => 'The snippet has been deleted',
				));
			}
			else
			{
				return $this->_send_object_not_found('snippet', $template_id);
			}
		}
		else if ($template_group_id == 'variables')
		{
			$global_var = $this->ee_template_lib->get_global_variable($template_id);
			if ($global_var)
			{
				$global_var->delete();
				return ee()->output->send_ajax_response(array(
					'success' => TRUE,
					'message' => 'The Global Variable has been deleted',
				));
			}
			else
			{
				return $this->_send_object_not_found('global variable', $template_id);
			}
		}
		else
		{
			// Get the template, check if it's parent is the correct one
			$template = $this->ee_template_lib->get_template($template_id);
			if ($template)
			{
				if ($template->group_id == $template_group_id)
				{
					$template->delete();
					return ee()->output->send_ajax_response(array(
						'success' => TRUE,
						'message' => 'The template has been deleted',
					));
				}
				else
				{
					// Send Bad request http code
					set_status_header(400);
					return ee()->output->send_ajax_response(array(
						'success' => FALSE,
						'message' => 'That template doesn\'t belong to that template group',
					));
				}
			}
			else
			{
				return $this->_send_object_not_found('template', $template_id);
			}
		}

	}

	public function rename_template_group()
	{
		$this->_init_check();

		$template_group_id = intval(ee()->input->post('template_group_id'));
		$template_group_name = ee()->input->post('template_group_name', TRUE);
		$template_group = $this->ee_template_lib->get_template_group($template_group_id);

		if ($template_group)
		{
			if ( ! preg_match("#^[a-zA-Z0-9_\-/]+$#i", $template_group_name))
			{
				return $this->_send_name_invalid('template group');
			}

			$template_group->group_name = $template_group_name;
			// This fails in EE 3.2.1 -- EE bug, not the add-on :D -> 'Unknown validation rule `is_valid_group_name`.'
			//$result = $template_group->validate();

			$template_group->save();
			return ee()->output->send_ajax_response(array(
				'success' => TRUE,
				'message' => 'The template group has been renamed.',
				'template_group' => $this->mountee_lib->convert_to_mountee_array($template_group)
			));

			if ($result->isValid())
			{
				$template_group->save();
				return ee()->output->send_ajax_response(array(
					'success' => TRUE,
					'message' => 'The template group has been renamed.',
					'template_group' => $this->mountee_lib->convert_to_mountee_array($template_group)
				));
			}
			else
			{
				return $this->_send_object_invalid($result, 'template group');
			}
		}
		else
		{
			return $this->_send_object_not_found('template group', $template_group_id);
		}

	}

	public function rename_template()
	{
		$this->_init_check();

		$template_id = intval(ee()->input->post('template_id'));
		$template_group_id = ee()->input->post('template_group_id');
		$template_name = ee()->input->post('template_name', TRUE);

		$extension = pathinfo($template_name, PATHINFO_EXTENSION);
		$template_type = Mountee_lib::get_type_from_extension($extension);

		// Remove extension from template name
		if ($extension != '')
		{
			$template_name = str_replace('.'.$extension, '', $template_name);
		}
		else
		{
			// Default type is webpage, php file type
			$extension = 'php';
		}

		if ($template_group_id == NULL)
		{
			// Special case when we come from the move_template endpoint
			$template_group_id = ee()->input->post('from_group_id');
		}

		// Added a space ' ' for convenience in Mountee filesystem
		if ( ! preg_match("#^[a-zA-Z0-9_\ \-/]+$#i", $template_name))
		{
			return $this->_send_name_invalid('template');
		}

		$template_name_temp = $template_name;
		if (strpos($template_name, ' ') !== FALSE)
		{
			// We accept spaces for convenience, but validation test would fail
			// So we replace spaces with tabs and add the timestamp to be sure it's unique
			$template_name_temp = str_replace(' ', '_', $template_name) . time();
		}

		if ($template_group_id == 'snippets')
		{
			$snippet = $this->ee_template_lib->get_snippet($template_id);
			if ($snippet)
			{
				if ($this->ee_template_lib->snippet_with_name_exists($template_name))
				{
					// Send Conflict http code
					set_status_header(409);
					return ee()->output->send_ajax_response(array(
						'success' => FALSE,
						'message' => 'The snippet name will conflict with an existing snippet.',
					));
				}

				$snippet->snippet_name = $template_name_temp;
				$result = $snippet->validate();
				if ($result->isValid())
				{
					$snippet->snippet_name = $template_name;
					$snippet->save();
					return ee()->output->send_ajax_response(array(
						'success' => TRUE,
						'message' => 'The snippet has been renamed.',
						'template' => $this->mountee_lib->convert_to_mountee_array($snippet)
					));
				}
				else
				{
					return $this->_send_object_invalid($result, 'snippet');
				}
			}
			else
			{
				return $this->_send_object_not_found('snippet', $template_id);
			}
		}
		else if ($template_group_id == 'variables')
		{
			$global_var = $this->ee_template_lib->get_global_variable($template_id);
			if ($global_var)
			{
				if ($this->ee_template_lib->global_variable_with_name_exists($template_name))
				{
					// Send Conflict http code
					set_status_header(409);
					return ee()->output->send_ajax_response(array(
						'success' => FALSE,
						'message' => 'The global variable name will conflict with an existing global variable.',
					));
				}

				$global_var->variable_name = $template_name_temp;
				$result = $global_var->validate();
				if ($result->isValid())
				{
					$global_var->variable_name = $template_name;
					$global_var->save();
					return ee()->output->send_ajax_response(array(
						'success' => TRUE,
						'message' => 'The global variable has been renamed.',
						'template' => $this->mountee_lib->convert_to_mountee_array($global_var)
					));
				}
				else
				{
					return $this->_send_object_invalid($result, 'global variable');
				}
			}
			else
			{
				return $this->_send_object_not_found('global variable', $template_id);
			}
		}
		else
		{
			$template = $this->ee_template_lib->get_template($template_id);

			if ($template)
			{
				// Check template type and change it if needed
				if ($template_type != '')
				{
					if (!in_array($template_type, $this->ee_template_lib->get_template_types()))
					{
						// Send Bad request http code
						set_status_header(400);
						return ee()->output->send_ajax_response(array(
							'success' => FALSE,
							'message' => 'The template type \''.$template_type.'\' isn\'t valid.',
						));
					}
					$template->template_type = $template_type;
				}

				// Check if template name would conflict
				$template_same_name = $this->ee_template_lib->get_template_from_name($template_group_id, $template_name);
				if ($template_same_name)
				{
					if ($template_same_name->template_id == $template->template_id)
					{
						// The conflict template is the template we're renaming
						// that means we might be trying to change only the extension
						if ($template_type != $template_same_name->template_type)
						{
							// Change the template type
							$template->template_type = $template_type;
							$template->save();
							return ee()->output->send_ajax_response(array(
								'success' => TRUE,
								'message' => 'The template type has been changed.',
								'template' => $this->mountee_lib->convert_to_mountee_array($template)
							));
						}
						else
						{
							// We don't need to change anything...
							// Not sure what to return
							return ee()->output->send_ajax_response(array(
								'success' => TRUE,
								'template' => $this->mountee_lib->convert_to_mountee_array($template)
							));
						}
						
					}
					else
					{
						// Name conflict
						// Send Conflict http code
						set_status_header(409);
						return ee()->output->send_ajax_response(array(
							'success' => FALSE,
							'message' => 'The template name will conflict with an existing template.',
						));
					}
				}

				$template->template_name = $template_name_temp;
				$result = $template->validate();
				if ($result->isValid())
				{
					$template->template_name = $template_name;
					$template->save();
					return ee()->output->send_ajax_response(array(
						'success' => TRUE,
						'message' => 'The template has been renamed.',
						'template' => $this->mountee_lib->convert_to_mountee_array($template)
					));
				}
				else
				{
					return $this->_send_object_invalid($result, 'template');
				}
			}
			else
			{
				return $this->_send_object_not_found('template', $template_id);
			}
		}
		
	}

	private function _create_snippet()
	{
		$snippet_name = ee()->input->post('template_name', TRUE);
		$template_data = ee()->input->post('template_data', TRUE);

		$extension = pathinfo($snippet_name, PATHINFO_EXTENSION);

		// Remove extension from template name
		if ($extension != '')
		{
			$snippet_name = str_replace('.'.$extension, '', $snippet_name);
		}

		if ($this->ee_template_lib->snippet_with_name_exists($snippet_name))
		{
			// Send Conflict http code
			set_status_header(409);
			return ee()->output->send_ajax_response(array(
				'success' => FALSE,
				'message' => 'The snippet name will conflict with an existing snippet.',
			));
		}

		$snippet_name_temp = $snippet_name;
		if (strpos($snippet_name, ' ') !== FALSE)
		{
			// We accept spaces for convenience, but validation test would fail
			// So we replace spaces with tabs and add the timestamp to be sure it's unique
			$snippet_name_temp = str_replace(' ', '_', $snippet_name) . time();
		}

		$snippet = ee('Model')->make('Snippet', array(
			'snippet_name' => $snippet_name_temp,
			'site_id'		=> ee()->config->item('site_id')
		));

		if ($template_data != "")
		{
			$snippet->snippet_contents = base64_decode($template_data);
		}

		$result = $snippet->validate();
		if ($result->isValid())
		{
			$snippet->snippet_name = $snippet_name;
			$snippet->save();
			return ee()->output->send_ajax_response(array(
				'success' => TRUE,
				'message' => 'The snippet has been created.',
				'template' => $this->mountee_lib->convert_to_mountee_array($snippet)
			));
		}
		else
		{
			return $this->_send_object_invalid($result, 'snippet');
		}

	}

	private function _create_global_variable()
	{
		$global_var_name = ee()->input->post('template_name', TRUE);
		$template_data = ee()->input->post('template_data', TRUE);

		$extension = pathinfo($global_var_name, PATHINFO_EXTENSION);

		// Remove extension from template name
		if ($extension != '')
		{
			$global_var_name = str_replace('.'.$extension, '', $global_var_name);
		}

		if ($this->ee_template_lib->global_variable_with_name_exists($global_var_name))
		{
			// Send Conflict http code
			set_status_header(409);
			return ee()->output->send_ajax_response(array(
				'success' => FALSE,
				'message' => 'The global variable name will conflict with an existing global variable.',
			));
		}

		$global_var_name_temp = $global_var_name;
		if (strpos($global_var_name, ' ') !== FALSE)
		{
			// We accept spaces for convenience, but validation test would fail
			// So we replace spaces with tabs and add the timestamp to be sure it's unique
			$global_var_name_temp = str_replace(' ', '_', $global_var_name) . time();
		}

		$global_var = ee('Model')->make('GlobalVariable', array(
			'variable_name' => $global_var_name_temp,
			'site_id'		=> ee()->config->item('site_id')
		));

		if ($template_data != "")
		{
			$global_var->variable_data = base64_decode($template_data);
		}

		$result = $global_var->validate();
		if ($result->isValid())
		{
			$global_var->variable_name = $global_var_name;
			$global_var->save();
			return ee()->output->send_ajax_response(array(
				'success' => TRUE,
				'message' => 'The global variable has been created.',
				'template' => $this->mountee_lib->convert_to_mountee_array($global_var)
			));
		}
		else
		{
			return $this->_send_object_invalid($result, 'global variable');
		}
	}

	/**
	 * Initial check before doing any actions
	 */
	private function _init_check()
	{
		if (ee()->session->userdata('member_id') !== 0)
		{
			// Everything's good
		}
		else
		{
			// Send Unauthorized code
			set_status_header(401);
			// Logged out, send out the response directly (code won't go further)
			return ee()->output->send_ajax_response(array(
				'logged_in' => FALSE,
				'message' => 'You need to be logged in',
			));
		}

		// Check the save template as files EE option
		if (ee()->config->item('save_tmpl_files') != 'n')
		{
			set_status_header(409); //Conflict
			return ee()->output->send_ajax_response(array(
				'logged_in' => TRUE,
				'message' => 'You can\'t use Mountee when save template as files is enabled',
			));
		}

		// Check if we're trying to modify a file, if it's the dummy file, don't do anything
		$template_id = ee()->input->post('template_id', TRUE);
		if ($template_id == Mountee_lib::$dum_file_id)
		{
			// Return empty response
			return ee()->output->send_ajax_response(array('success' => false));
		}
	}

	private function _send_object_not_found($object_name = "", $object_id = NULL)
	{
		if ($object_name == "")
		{
			$object_name = "object";
		}
		$object_id_str = "";
		if ($object_id != NULL)
		{
			$object_id_str = " with id ".$object_id;
		}
		set_status_header(400);
		return ee()->output->send_ajax_response(array(
			'success' => FALSE,
			'message' => 'Cannot find '.$object_name.$object_id_str,
		));
	}

	private function _send_name_invalid($object_name = "")
	{
		if ($object_name == "")
		{
			$object_name = "object";
		}
		// Send Bad request http code
		set_status_header(400);
		return ee()->output->send_ajax_response(array(
			'success' => FALSE,
			'message' => 'The '.$object_name.' name is invalid.',
		));
	}

	private function _send_object_invalid($validation_result, $object_name = 'object')
	{
		$error_str = "";
		foreach ($validation_result->getAllErrors() as $error_key => $error_messages)
		{
			$error_str .= $error_key.': ';
			foreach ($error_messages as $error_message)
			{
				$error_str .= $error_message.',';
			}
			$error_str = substr($error_str, 0, -1);
			$error_str .= '; ';
		}
		// Send Bad request http code
		set_status_header(400);
		return ee()->output->send_ajax_response(array(
			'success' => FALSE,
			'message' => 'The '.$object_name.' couldn\'t be validated. '.$error_str,
		));
	}

	private function _send_permissions_insufficient($action = '')
	{
		if ($action != '')
		{
			$action = ' for ' . $action;
		}
		set_status_header(403);
		return ee()->output->send_ajax_response(array(
			'success' => FALSE,
			'message' => 'You don\'t have sufficient rights' . $action,
		));
	}


	//**********************************************
	// ExpressionEngine Own stuff (copy/pasted)
	//**********************************************

	private function _do_auth($username, $password)
	{
		$sess = ee()->auth->authenticate_username($username, $password);

		if ( ! $sess)
		{
			ee()->session->save_password_lockout($username);

			if (empty($username) OR empty($password))
			{
				ee()->output->send_ajax_response(array(
				'logged_in' => FALSE,
				'message' => 'missing auth info',
			));
			}
			else
			{
				ee()->output->send_ajax_response(array(
				'logged_in' => FALSE,
				'message' => 'invalid auth info',
			));
			}
		}

		// Banned
		if ($sess->is_banned())
		{
			ee()->output->send_ajax_response(array(
				'logged_in' => FALSE,
				'message' => 'not authorized',
			));
		}

		// Allow multiple logins?
		// Do we allow multiple logins on the same account?
		if (ee()->config->item('allow_multi_logins') == 'n')
		{
			if ($sess->has_other_session())
			{
				ee()->output->send_ajax_response(array(
					'logged_in' => FALSE,
					'message' => 'not authorized',
				));
			}
		}

		// Check user/pass minimum length
		$this->_check_min_unpwd($sess, $username, $password);

		$csrf_token = ee()->csrf->refresh_token();

		// Start Session
		// "Remember Me" is one year
		if (isset($_POST['auto_login']))
		{
			$sess->remember_me();
		}

		$anon = (ee()->input->post('anon') == 1) ? FALSE : TRUE;

		$sess->anon($anon);

		$sess->start_session();

		return $sess;
	}

	private function _do_multi_auth($sites, $login_state)
	{
		if ( ! $sites
			OR ee()->config->item('allow_multi_logins') == 'n'
			OR empty($login_state))
		{
			return ee()->output->show_user_error('general', lang('not_authorized'));
		}

		// Kill old sessions first
		ee()->session->gc_probability = 100;
		ee()->session->delete_old_sessions();

		// Grab session
		$sess_q = ee()->db->get_where('sessions', array(
			'user_agent'  => substr(ee()->input->user_agent(), 0, 120),
			'login_state' => $login_state
		));

		if ( ! $sess_q->num_rows())
		{
			return ee()->output->show_user_error('general', lang('not_authorized'));
		}

		// Grab member
		$mem_q = ee()->db->get_where('members', array(
			'member_id' => $sess_q->row('member_id')
		));

		if ( ! $mem_q->num_rows())
		{
			return FALSE;
		}

		$incoming = new Auth_result($mem_q->row());
		$csrf_token = ee()->csrf->refresh_token();

		// this is silly - only works for the first site
		if (isset($_POST['auto_login']))
		{
			$incoming->remember_me();
		}

		// hook onto an existing session
		$incoming->use_session_id($sess_q->row('session_id'));
		$incoming->start_session();

		$new_row = $sess_q->row_array();
		$some_row['site_id'] = ee()->config->item('site_id');

		return $incoming;
	}

	private function _check_min_unpwd($member_obj, $username, $password)
	{
		$uml = ee()->config->item('un_min_len');
		$pml = ee()->config->item('pw_min_len');

		$ulen = strlen($username);
		$plen = strlen($password);

		if ($ulen < $uml OR $plen < $pml)
		{
			$trigger = '';
			if (ee()->input->get_post('FROM') == 'forum')
			{
				$this->basepath = ee()->input->get_post('mbase');
				$trigger = ee()->input->get_post('trigger');
			}

			$path = 'unpw_update/' . $member_obj->member('member_id') . '_' . $ulen . '_' . $plen;

			if ($trigger != '')
			{
				$path .= '/'.$trigger;
			}

			return ee()->functions->redirect($this->_member_path($path));
		}
	}
}
